export default {
  methods: {
    initGenerationModels(models, options) {
      // NOTE: create local selected list & refs !!important
      models.forEach(gen => {
        const localSelected = gen.models?.filter(this.isGroupSelected)
        localSelected?.sort((l, r) => l.selectedRank - r.selectedRank)
        gen.selectedList = localSelected || []
        gen.selectedListSnapshot = this.deepClone(gen.selectedList) // 快照，用于对比变更
        gen.option = Object.values(options).find(opt => opt.value == gen.generation)
        gen.prevModel = models.find(m => m.generation == gen.generation - 1)
        gen.nextModel = models.find(m => m.generation == gen.generation + 1)
      })
    },
    isModelEnrolled(model) {
      // 代录取
      return model.models?.some(this.isGroupEnrolled)
    },
    isModelSelected(model) {
      // 代组合填报
      return model.models?.some(this.isGroupSelected)
    },
    isModelRejected(model) {
      return model.models?.some(this.isGroupRejected)
    },
    isModelApproved(model) {
      // 代正常录取
      return model.models?.some(this.isGroupApproved)
    },
    isModelForceAdjusted(model) {
      // 代强制调剂
      return model.models?.some(this.isGroupForceAdjusted)
    },
    isGroupEnrolled(group) {
      // 组合录取 = 正常录取 | 强制调剂
      return group.approved || group.forceAdjusted
    },
    isGroupSelected(group) {
      // 组合填报 = 同意 | 改选 | 拒绝
      return group.selected || group.rejected
    },
    isGroupRejected(group) {
      // 组合填报拒绝
      return group.rejected
    },
    isGroupApproved(group) {
      // 组合正常录取
      return group.approved
    },
    isGroupForceAdjusted(group) {
      // 组合强制调剂
      return group.forceAdjusted
    },
    combineGroupStatus(group, model) {
      // 逻辑同elective-preference-command
      if(this.isGroupEnrolled(group)) return '已录取'
      if (!group.allowSelect) return group.disabledReason || '无法报名'
      if (this.isModelRejected(model)) return '已拒绝'
      if (group.selected) return '已报名'
      return '报名中'
    }
  }
}
